import {EmblaCarouselType} from 'embla-carousel'
import {BreakpointKey, BREAKPOINTS} from "../config/responsiveConfig"

export const addPrevNextBtnsClickHandlers = (
    carouselApi: EmblaCarouselType,
    prevBtn?: Element | null,
    nextBtn?: Element | null,
): (() => void) => {
  const scrollPrev = () => carouselApi.scrollPrev()
  const scrollNext = () => carouselApi.scrollNext()
  prevBtn?.addEventListener('click', scrollPrev, false)
  nextBtn?.addEventListener('click', scrollNext, false)

  return (): void => {
    prevBtn?.removeEventListener('click', scrollPrev, false)
    nextBtn?.removeEventListener('click', scrollNext, false)
  }
}

export const addTogglePrevNextBtnsActive = (
    carouselApi: EmblaCarouselType,
    prevBtn?: Element | null,
    nextBtn?: Element | null,
): (() => void) => {
  const togglePrevNextBtnsState = (): void => {
    carouselApi.canScrollPrev()
        ? prevBtn?.removeAttribute('disabled')
        : prevBtn?.setAttribute('disabled', 'disabled')

    carouselApi.canScrollNext()
        ? nextBtn?.removeAttribute('disabled')
        : nextBtn?.setAttribute('disabled', 'disabled')
  }

  carouselApi
      .on('select', togglePrevNextBtnsState)
      .on('init', togglePrevNextBtnsState)
      .on('reInit', togglePrevNextBtnsState)

  return (): void => {
    prevBtn?.removeAttribute('disabled')
    nextBtn?.setAttribute('disabled', 'disabled')
  }
}

export const addDotBtnsAndClickHandlers = (
    carouselApi: EmblaCarouselType,
    dotsNode?: Element | null,
): (() => void) => {
  let dotNodes: HTMLElement[] = []

  const addDotBtnsWithClickHandlers = (): void => {
    if (dotsNode) {
      dotsNode.innerHTML = carouselApi
          .scrollSnapList()
          .map(() => '<button class="dot" type="button"></button>')
          .join('')

      dotNodes = Array.from(dotsNode.querySelectorAll('.dot'))
      dotNodes.forEach((dotNode, index) => {
        dotNode.addEventListener('click', () => carouselApi.scrollTo(index), false)
      })
    }
  }

  const toggleDotBtnsActive = (): void => {
    const previous = carouselApi.previousScrollSnap()
    const selected = carouselApi.selectedScrollSnap()
    dotNodes[previous]?.classList.remove('selected')
    dotNodes[selected]?.classList.add('selected')
  }

  carouselApi
      .on('init', addDotBtnsWithClickHandlers)
      .on('reInit', addDotBtnsWithClickHandlers)
      .on('init', toggleDotBtnsActive)
      .on('reInit', toggleDotBtnsActive)
      .on('select', toggleDotBtnsActive)

  return (): void => {
    if (dotsNode) dotsNode.innerHTML = ''
  }
}

export const createDotsAndArrows = (carouselNode?: HTMLDivElement,
                                    carouselApi?: EmblaCarouselType) => {
  if (carouselNode && carouselApi) {
    const prevBtn = carouselNode.querySelector('.prev')
    const nextBtn = carouselNode.querySelector('.next')
    const dotsNode = carouselNode.querySelector('.dots')

    const removePrevNextBtnsClickHandlers = addPrevNextBtnsClickHandlers(carouselApi, prevBtn, nextBtn)
    const removeTogglePrevNextBtnsActive = addTogglePrevNextBtnsActive(carouselApi, prevBtn, nextBtn)
    const removeDotBtnsAndClickHandlers = addDotBtnsAndClickHandlers(carouselApi, dotsNode)

    carouselApi
        .on('destroy', removePrevNextBtnsClickHandlers)
        .on('destroy', removeTogglePrevNextBtnsActive)
        .on('destroy', removeDotBtnsAndClickHandlers)
  }
}

/**
 *  breakpoint에 해당하는 속성이 없다면 이전 화면의 field를 가져온다.
 *  breakpoint가 [xl, lg, md, sm] 일 때, xl에 field가 없다면 lg | md | sm 의 값을 가져온다.
 */
export const findNextBreakpointProp = (prop: keyof UiCarouselProps,
                                       propsBreakpoints?: UiCarouselProps['breakpoints'],
                                       breakpointKey?: BreakpointKey) => {

  const breakpointsOrder = <BreakpointKey[]>Object.keys(BREAKPOINTS).reverse()
  const findIndex = () => breakpointsOrder.findIndex((keys) => keys === breakpointKey)
  const getField = (index: number) => propsBreakpoints?.[breakpointsOrder[index]]?.[prop]

  for (let i = findIndex(); i < breakpointsOrder.length; i++) {
    if (i === -1) return
    const value = getField(i)
    if (value) return value
  }
}

/**
 *  UiCarousel에서 breakpoints: {lg: {arrow: true}} 일 때,
 *  현재 화면이 lg 이상일 때, arrow: true 를 리턴하고, lg이하일 때, 기본 props값을 리턴한다.
 */
export const getBreakpointsProp = (prop: keyof UiCarouselProps,
                           props: UiCarouselProps,
                           breakpointKey?: BreakpointKey) => {
  const responsiveCounter = findNextBreakpointProp(prop, props.breakpoints, breakpointKey)
  return responsiveCounter ? responsiveCounter : props?.[prop]
}

export type UiCarouselProps = {
  perView?: number | 'auto'
  loop?: boolean
  center?: boolean
  gap?: string
  dragFree?: boolean
  slidesToScroll?: number
  autoplay?: boolean
  autoplayDelay?: number
  vertical?: boolean
  arrow?: boolean
  dots?: boolean
  counter?: boolean
  startIndex?: number
  breakpoints?: { [key in BreakpointKey]?: UiCarouselProps }
}