/**
 * PreventCopy directive
 *
 * element 의 오른쪽 마우스 버튼을 막고 (oncontextmenu 처리),
 * 선택/드래그 되지 않도록 설정한다. (user-select-none 스타일)
 *
 * ```
 * import preventCopy from "@/directive/preventCopy"
 *
 * export default defineComponent({
 *   directives: {preventCopy},
 * })
 * ```
 *
 * ```
 * <template>
 *   <img v-prevent-copy/>
 *   <img v-prevent-copy="true"/>
 * </template>
 * ```
 */

import {Directive, DirectiveBinding} from "vue"

const preventDefaultHandler = (event: Event) => event.preventDefault()

export default {
  updated: (element: Element, {value}: DirectiveBinding): void => {
    if (value == undefined || value) {

      // oncontextmenu 이벤트가 동작하지 않도록 만든다. 이벤트 핸들러는 한 번만 등록한다.
      if (element.getAttribute('prevent-copy-handler-registered') !== 'true') {
        element.addEventListener('contextmenu', preventDefaultHandler)
        element.setAttribute('prevent-copy-handler-registered', 'true')
      }

      // 선택 후 ctrl-c 가 동작하지 못하도록, use-select: none 스타일을 추가한다.
      if (!element.classList.contains('user-select-none')) {
        element.classList.add('user-select-none')
      }
    }
  }
} as Directive