import {PaginatedList, PaginatedListLoadInfo} from "./PaginatedList"

/**
 * 여러 개의 PaginatedList 가 모여 구성된 전체 목록.
 *
 * PaginatedList 가 중간에 건너뛰지 않고 순서대로 추가되기를 기대하지만,
 * 만약 중간에 건너뛴 경우, undefined 로 채워져있는 것으로 취급된다.
 */
export class TotalList<T> {

  data: T[] = []
  totalCount = -1
  loadedCount = -1

  /**
   * `TotalList`에 `PaginatedList`를 추가한다.
   * @param list 추가할 `PaginatedList`
   */
  append(list?: PaginatedList<T>): void {
    if (list) {
      // totalCount 는 최신의 것을 따른다.
      this.totalCount = list.totalCount
      // list 가 제일 끝이 아닐 수 있으므로, 큰 값을 따른다.
      this.loadedCount = Math.max(list.offset + list.data.length, this.loadedCount)
      list.data.forEach((value, idx) => {
        this.data[list.offset + idx] = value
      })
    }
  }

  /**
   * @return `TotalList`에 `PaginatedList`가 한 번도 추가된 적이 없으면 true, 그렇지 않으면 false
   */
  isBeforeInitialized(): boolean {
    return this.totalCount < 0
  }

  /**
   * @return 데이터가 없으면 true. 초기화 전에는 empty 여부를 판단할 수 없으므로, false 를 리턴한다.
   */
  isEmpty(): boolean {
    return this.totalCount === 0
  }

  /**
   * `TotalList`를 초기화 전 상태로 되돌린다.
   */
  clear(): void {
    this.data.length = 0
    this.totalCount = -1
    this.loadedCount = -1
  }

  /**
   * @return 더 로드할 데이터가 남아있으면 true, 그렇지 않으면 false. 초기화 전에는 false.
   */
  isMoreDataAvailable(): boolean {
    return (this.totalCount >= 0) && (this.totalCount > this.loadedCount)
  }

  static copyOf<T>(src: TotalList<T>): TotalList<T> {
    const copy = new TotalList<T>()
    copy.data = [...src.data]
    copy.totalCount = src.totalCount
    copy.loadedCount = src.loadedCount
    return copy
  }
}

export type TotalListLoadInfo<T> = PaginatedListLoadInfo & {
  totalLoaded?: number
  totalList?: TotalList<T>
}